var fs = require('fs');
var Canvas = require("canvas"),
	Image = Canvas.Image,
	gm = require('gm'),
    path = require("path");
var mkdirp = require('mkdirp');
var SceneImage = require("./ui/SceneImage"),
    Bounds = require('shardedmapview').Bounds,
    canvasToPng8 = require('./canvasToPng8');

const DecimalPointToPoint = require('./DecimalPointToPoint');

const Decimal = require('shardedmapview').DecimalConfigured;
const D = num => new Decimal(num);

module.exports = function TileImage({tile, imageExporter}) {

	const tileImagePixelWidth = 512,
	      tileImagePixelHeight = tileImagePixelWidth;

	var canvas = new Canvas(tileImagePixelWidth, tileImagePixelHeight),
	    ctx = canvas.getContext('2d');
	ctx.antialias = 'subpixel';
	ctx.patternQuality = 'best';
	ctx.filter = 'best';

	const loadBitmap = function(sceneImage, cb) {
		fs.readFile(
			path.join(process.cwd(), sceneImage.imageUrl),
			function(err, sceneImageData) {
				if(err) {
					cb(err);
				}
				else {
					var sceneImageBitmap = new Image;
					sceneImageBitmap.src = sceneImageData;
					cb(null, sceneImageBitmap);
				}
			}
		);
	}

	const getProjectionInfo = function(sceneImage, sceneImageBitmap) {
		var sceneImageBitmapPixelDiagonalDistance = Math.sqrt(Math.pow(sceneImageBitmap.width, 2) + Math.pow(sceneImageBitmap.height, 2));
		// console.log({sceneImageBitmapPixelDiagonalDistance});
		var ratioTileToWorld = D(tileImagePixelWidth).dividedBy(tile.width());
		var sceneImageDiagonalDistanceInTilePixels = ratioTileToWorld.times(SceneImage.DiagonalDistance(sceneImage)).toNumber();
		var scale = sceneImageDiagonalDistanceInTilePixels / sceneImageBitmapPixelDiagonalDistance;
		return {
			scale,
			diagonalDistanceInTilePixels: sceneImageDiagonalDistanceInTilePixels
		};
	}

	const bitmapImageDirectoryPath = function() {
		return path.join(
			'tiles',
			''+tile.zoom,
			''+tile.row.toString()
		);
	}

	const bitmapImagePath = function() {
		return path.join(bitmapImageDirectoryPath(), tile.column.toString() + '.' + imageExporter.imageExtension());
	}

	return {

		sceneImageProjectionInfo: function(sceneImage, cb) {
			loadBitmap(sceneImage, function(err, sceneImageBitmap) {
				if(err) {
					cb(err);
				}
				else {
					cb(null, getProjectionInfo(sceneImage, sceneImageBitmap));
				}
			});
		},

		drawParentQuadrant: function(cb) {
		    const parentTile = tile.getParent();
		    TileImage({tile: parentTile, imageExporter}).loadTileImageBitmap((err, parentTileImageBitmap) => {

		    	if(err) {
		    		cb(err);
		    	}
		    	else {
				    const quadrantX = parentTile.column.times(2).minus(tile.column).toNumber(),
				          quadrantY = parentTile.row.times(2).minus(tile.row).toNumber();

				    try {
					    ctx.drawImage(
					    	parentTileImageBitmap,
					    	quadrantX * (tileImagePixelWidth) - 0.5,
					    	quadrantY * (tileImagePixelHeight) - 0.5,
					    	tileImagePixelWidth * 2 + 1,
					    	tileImagePixelHeight * 2 + 1
					    );
					    //console.log('drew parent quadrant from ' + parentTile.key());
				    }
				    catch(e) {
				    	console.error('Could not draw parent ' + parentTile.key());
				    }

				    

				    cb(null);
		    	}

		    });
		},

		drawSceneImage: function(sceneImage, cb) {

			loadBitmap(sceneImage, function(err, sceneImageBitmap) {
				if(err) {
					cb(err);
				}
				else {
					var rotation = -SceneImage.GetRotation(sceneImage);

					var sceneImageUpperLeft = SceneImage.GetUpperLeft(sceneImage);
					const transform = tile.bounds().transformerTo(Bounds({
						left: 0,
						top: 0,
						bottom: tileImagePixelHeight,
						right: tileImagePixelWidth
					}));
					var sceneImageUpperLeftInTilePixelSpace = DecimalPointToPoint(
						transform(sceneImageUpperLeft)
					);
					
					
					var x = sceneImageUpperLeftInTilePixelSpace.x,
					    y = sceneImageUpperLeftInTilePixelSpace.y;

					const {scale} = getProjectionInfo(sceneImage, sceneImageBitmap);

					ctx.translate(x, y);
					ctx.rotate(rotation);
					ctx.drawImage(sceneImageBitmap,
						0, 0,
						sceneImageBitmap.width * scale,
						sceneImageBitmap.height * scale
					);
					ctx.resetTransform();
					
					cb();
				}
			});

		},
		save: function(cb) {
			mkdirp(bitmapImageDirectoryPath(), function (err) {
			    if (err) {
			    	cb(err);
			    }
			    else {
			    	
			    	imageExporter.export(canvas, bitmapImagePath(), err => {
			    		if(err) {
			    			cb(cb);
			    		}
			    		else {
			    			cb(null);
			    		}
			    	});

			    }
			});
			
		},
		loadTileImageBitmap: function(cb) {
			fs.readFile(bitmapImagePath(), (err, data) => {
				if(err) {
					cb(err);
				}
				else {
					const img = new Image;
					img.src = data;
					cb(null, img);
				}
			});
		},
		bitmapImagePath
	};
}
