#!/usr/bin/env node

const argv = require('yargs').argv;
const mkdirp = require('mkdirp');
const path = require('path');
const async = require('async');
const fs = require('fs');
const _ = require('lodash');
const Datauri = require('datauri');

const TileImage = require('./TileImage');

const {DecimalConfigured, Tile, Bounds} = require('shardedmapview');
const Decimal = DecimalConfigured;
const D = num => new Decimal(num);

const sceneFilePath = path.join(process.cwd(), 'scene.json');

const imageExporters = {
	'png8': require('./canvasToPng8'),
	'jpeg': require('./canvasToJpeg')
};

module.exports = {
	usage: {
		commandLine: () => 'xxx.xxxxxxxxxx,yyyy.yyyyyyyyyyyy',
		description: () => 'Bundle tiles at all zoom levels containing given point'
	},
	run: (cliArgs, {onUsageError}) => {

		const imageExporterKey = argv.exporter;
		const imageExporter = imageExporters[imageExporterKey];
		console.error('image exporter is ' + imageExporterKey + '. configure with --exporter');


		let point = cliArgs[0].split(',').map(num => new Decimal(num));
		point = {x: point[0], y: point[1]};

		const maxZoom = +argv.maxZoom || 70;
		const minZoom = +argv.minZoom || 0;

		console.error(`min zoom is ${minZoom}. Configure with --min-zoom`);
		console.error(`max zoom is ${maxZoom}. Configure with --max-zoom`);

		const paddingInTiles = +argv.padding || 0;
		console.error(`padding is ${paddingInTiles} tiles. Configure with --padding`);

		if(false) {
			onUsageError('Usage error goes here');
		}
		else {
			const _tiles = _.flatten(_.range(minZoom, maxZoom).map(zoom => {
				// return Tile.tileForPointAtZoom(point, zoom);
				const tileSize = Tile({zoom, row: 0, column: 0}).width();
				const padding = D(paddingInTiles).times(tileSize);
				const bounds = Bounds({
					left:   D(point.x).minus(padding),
					right:  D(point.x).plus( padding),
					top:    D(point.y).minus(padding),
					bottom: D(point.y).plus( padding)
				});
				return Tile.tilesForBoundsAtZoom(bounds, zoom);
			}));

			let tileBundlesByZoom = {};
			async.map(
				_tiles,
				(tile, cb) => {
					const tileImage = TileImage({tile, imageExporter});


					const datauri = new Datauri();
					 
					datauri.on('encoded', content => {
						tileBundlesByZoom[tile.zoom] = tileBundlesByZoom[tile.zoom] || {};
						tileBundlesByZoom[tile.zoom][tile.key()] = content;
						cb(null, {[tile.key()]: content})
					});
					//=> "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAA..."; 
					 
					datauri.on('error', err => {
						if(err.code === 'ENOENT') {
							console.error(`Warning: ${tileImage.bitmapImagePath()} does not exist. This may be what you want.`);
							cb(null, {[tile.key()]: null});
						}
						else {
							cb(err);
						}
						
					});
					datauri.encode(tileImage.bitmapImagePath());

				},
				(err, tiles) => {
					if(err) {
						console.error(err);
					}
					else {
						console.log(JSON.stringify(_.merge.apply(_, tiles)));
						mkdirp('preload', err => {
							if(err) {
								console.error(err);
							}
							else {
								async.each(
									Object.keys(tileBundlesByZoom),
									(zoom, cb) => {
										fs.writeFile(`preload/zoom-${zoom}.json`, JSON.stringify(tileBundlesByZoom[zoom]), {encoding:'utf8'}, err => {
											cb(err);
										});
									},
									err => {
										if(err) {
											console.error(err);
										}
									}
								);
							}
						});
					}
					
				}
			);
		}


	}
}
